@@ -13,6 +13,7 @@ GNU General Public License for more details.'''
 
 import numpy as np
 import neuron
+
 from time import time
 
 def _run_simulation(cell, variable_dt=False, atol=0.001):
@@ -60,14 +61,14 @@ def _run_simulation(cell, variable_dt=False, atol=0.001):
         counter += 1.
         if np.mod(counter, interval) == 0:
             rtfactor = (neuron.h.t - ti)  * 1E-3 / (time() - t0)
-            print 't = %.0f, realtime factor: %.3f' % (neuron.h.t, rtfactor)
+            print('t = %.0f, realtime factor: %.3f' % (neuron.h.t, rtfactor))
             t0 = time()
             ti = neuron.h.t
 
 def _run_simulation_with_electrode(cell, electrode=None,
                                    variable_dt=False, atol=0.001,
                                    to_memory=True, to_file=False,
-                                   file_name=None, electrodecoeffs=None):
+                                   file_name=None, dotprodcoeffs=None):
     '''
     Running the actual simulation in NEURON.
     electrode argument used to determine coefficient
@@ -76,33 +77,36 @@ def _run_simulation_with_electrode(cell, electrode=None,
     try:
         import h5py
     except:
-        print 'h5py not found, LFP to file not possible'
+        print('h5py not found, LFP to file not possible')
         to_file = False
         file_name = None
 
     # Use electrode object(s) to calculate coefficient matrices for LFP
     # calculations. If electrode is a list, then
     if cell.verbose:
-        print 'precalculating geometry - LFP mapping'
+        print('precalculating geometry - LFP mapping')
         
-    
-    #assume now that if electrodecoeff != None, we're not using
-    #the electrode object:
-    if electrode != None and electrodecoeffs != None:
-        ERR = "electrode and electrodecoeffs can't both be != None"
-        raise AssertionError, ERR
     #put electrodecoeff in a list, if it isn't already
-    if electrodecoeffs != None:
-        if type(electrodecoeffs) != type(list):
-            electrodecoeffs = [electrodecoeffs]
+    if dotprodcoeffs != None:
+        if type(dotprodcoeffs) != list:
+            dotprodcoeffs = [dotprodcoeffs]
+        electrodes = []
     else:
+        #create empty list if no dotprodcoeffs are supplied
+        dotprodcoeffs = []
+    
+    #just for safekeeping
+    lendotrodcoeffs0 = len(dotprodcoeffs)
+    
+    #access electrode object and append dotprodcoeffs
+    if electrode != None:
         #put electrode argument in list if needed
-        if type(electrode) == type([]):
+        if type(electrode) == list:
             electrodes = electrode
         else:
             electrodes = [electrode]
         
-        #calculate list of electrodecoeffs, will try temp store of imem, tvec, LFP
+        #calculate list of dotprodcoeffs, will try temp store of imem, tvec, LFP
         cellTvec = cell.tvec
         try:
             cellImem = cell.imem.copy()
@@ -111,7 +115,6 @@ def _run_simulation_with_electrode(cell, electrode=None,
         
         cell.imem = np.eye(cell.totnsegs)
         cell.tvec = np.arange(cell.totnsegs) * cell.timeres_python
-        electrodecoeffs = []
         electrodeLFP = []   #list of electrode.LFP objects if they exist
         restoreLFP = False
         restoreCellLFP = False
@@ -124,7 +127,7 @@ def _run_simulation_with_electrode(cell, electrode=None,
                 CellLFP = el.CellLFP
                 restoreCellLFP = True
             el.calc_lfp(cell=cell)
-            electrodecoeffs.append(el.LFP.copy())
+            dotprodcoeffs.append(el.LFP.copy())
             if restoreLFP:
                 del el.LFP
                 el.LFP = LFPcopy
@@ -142,7 +145,10 @@ def _run_simulation_with_electrode(cell, electrode=None,
             cell.imem = cellImem
         except:
             del cell.imem
-    
+    elif electrode == None:
+        electrodes = None
+   
+
     # Initialize NEURON simulations of cell object    
     neuron.h.dt = cell.timeres_NEURON
     
@@ -187,7 +193,7 @@ def _run_simulation_with_electrode(cell, electrode=None,
     #LFPs for each electrode will be put here during simulation
     if to_memory:
         electrodesLFP = []
-        for coeffs in electrodecoeffs:
+        for coeffs in dotprodcoeffs:
             electrodesLFP.append(np.empty((coeffs.shape[0],
                                     cell.tstopms / cell.timeres_NEURON + 1)))
     #LFPs for each electrode will be put here during simulations
@@ -197,40 +203,39 @@ def _run_simulation_with_electrode(cell, electrode=None,
             file_name += '.h5'
         el_LFP_file = h5py.File(file_name, 'w')
         i = 0
-        for coeffs in electrodecoeffs:
+        for coeffs in dotprodcoeffs:
             el_LFP_file['electrode%.3i' % i] = np.empty((coeffs.shape[0],
                                     cell.tstopms / cell.timeres_NEURON + 1))
             i += 1
     
+    #multiply segment areas with specific membrane currents later,
+    #mum2 conversion factor:
+    area = cell.area * 1E-2
     #run fadvance until time limit, and calculate LFPs for each timestep
-    area = cell.area
     while neuron.h.t < cell.tstopms:
         if neuron.h.t >= 0:
             i = 0
             for sec in cell.allseclist:
                 for seg in sec:
-                    imem[i] = seg.i_membrane * area[i] * 1E-2
+                    imem[i] = seg.i_membrane
                     i += 1
-            #this is the lfp at this timestep
-            lfp = np.dot(coeffs, imem)
+            #pA/mum2 -> nA conversion
+            imem *= area
             
             if to_memory:
-                j = 0
-                for coeffs in electrodecoeffs:
-                    electrodesLFP[j][:, tstep] = lfp
-                    j += 1
+                for j, coeffs in enumerate(dotprodcoeffs):
+                    electrodesLFP[j][:, tstep] = np.dot(coeffs, imem)
                     
             if to_file:
-                j = 0
-                for coeffs in electrodecoeffs:
-                    el_LFP_file['electrode%.3i' % j][:, tstep] = lfp
+                for j, coeffs in enumerate(dotprodcoeffs):
+                    el_LFP_file['electrode%.3i' % j][:, tstep] = np.dot(coeffs, imem)
             
             tstep += 1
         neuron.h.fadvance()
         counter += 1.
-        if np.mod(counter, interval) == 0:
+        if divmod(counter, interval)[1] == 0:
             rtfactor = (neuron.h.t - ti) * 1E-3 / (time() - t0)
-            print 't = %.0f, realtime factor: %.3f' % (neuron.h.t, rtfactor)
+            print('t = %.0f, realtime factor: %.3f' % (neuron.h.t, rtfactor))
             t0 = time()
             ti = neuron.h.t
     
@@ -239,18 +244,17 @@ def _run_simulation_with_electrode(cell, electrode=None,
         i = 0
         for sec in cell.allseclist:
             for seg in sec:
-                imem[i] = seg.i_membrane * area[i] * 1E-2
+                imem[i] = seg.i_membrane
                 i += 1
+        #pA/mum2 -> nA conversion
+        imem *= area
+            
         if to_memory:
-            j = 0
-            for coeffs in electrodecoeffs:
-                lfp = np.dot(coeffs, imem)
-                electrodesLFP[j][:, tstep] = lfp
-                j += 1
+            for j, coeffs in enumerate(dotprodcoeffs):
+                electrodesLFP[j][:, tstep] = np.dot(coeffs, imem)
         if to_file:
-            j = 0
-            for coeffs in electrodecoeffs:
-                el_LFP_file['electrode%.3i' % j][:, tstep] = lfp
+            for j, coeffs in enumerate(dotprodcoeffs):
+                el_LFP_file['electrode%.3i' % j][:, tstep] = np.dot(coeffs, imem)
 
     except:
         pass
@@ -258,20 +262,22 @@ def _run_simulation_with_electrode(cell, electrode=None,
     # Final step, put LFPs in the electrode object, superimpose if necessary
     # If electrode.perCellLFP, store individual LFPs
     if to_memory:
-        j = 0
-        for el in electrodes:
-            if hasattr(el, 'LFP'):
-                el.LFP += electrodesLFP[j]
-            else:
-                el.LFP = electrodesLFP[j]
-            #will save each cell contribution separately
-            if el.perCellLFP:
-                if not hasattr(el, 'CellLFP'):
-                    el.CellLFP = []
-                el.CellLFP.append(electrodesLFP[j])
-            el.electrodecoeff = electrodecoeffs[j]
-            j += 1
-
+        #the first few belong to input dotprodcoeffs
+        cell.dotprodresults = electrodesLFP[:lendotrodcoeffs0]
+        #the remaining belong to input electrode arguments
+        if electrodes != None:
+            for j, LFP in enumerate(electrodesLFP):
+                if not j < lendotrodcoeffs0:
+                    if hasattr(electrodes[j-lendotrodcoeffs0], 'LFP'):
+                        electrodes[j-lendotrodcoeffs0].LFP += LFP
+                    else:
+                        electrodes[j-lendotrodcoeffs0].LFP = LFP
+                    #will save each cell contribution separately
+                    if electrodes[j-lendotrodcoeffs0].perCellLFP:
+                        if not hasattr(electrodes[j], 'CellLFP'):
+                            electrodes[j-lendotrodcoeffs0].CellLFP = []
+                        electrodes[j-lendotrodcoeffs0].CellLFP.append(LFP)
+                    electrodes[j-lendotrodcoeffs0].electrodecoeff = dotprodcoeffs[j]
 
     if to_file:
         el_LFP_file.close()
@@ -297,45 +303,95 @@ def _collect_geometry_neuron(cell):
     
     #loop over all segments
     for sec in cell.allseclist:
-        n3d = neuron.h.n3d()
+        n3d = int(neuron.h.n3d())
+        nseg = sec.nseg
+        gsen2 = 1./2/nseg
         if n3d > 0:
-            #length of sections
-            xlength = neuron.h.x3d(n3d - 1) - neuron.h.x3d(0)
-            ylength = neuron.h.y3d(n3d - 1) - neuron.h.y3d(0)
-            zlength = neuron.h.z3d(n3d - 1) - neuron.h.z3d(0)
+            #create interpolation objects for the xyz pt3d info:
+            L = np.zeros(n3d)
+            x = np.zeros(n3d)
+            y = np.zeros(n3d)
+            z = np.zeros(n3d)
+            for i in range(n3d):
+                L[i] = neuron.h.arc3d(i)
+                x[i] = neuron.h.x3d(i)
+                y[i] = neuron.h.y3d(i)
+                z[i] = neuron.h.z3d(i)
             
-            for seg in sec:
+            #normalize as seg.x [0, 1]
+            L /= sec.L
+                        
+            #temporary store position of segment midpoints
+            segx = np.zeros(nseg)
+            for i, seg in enumerate(sec):
+                segx[i] = seg.x
+            
+            #can't be >0 which may happen due to NEURON->Python float transfer:
+            segx0 = (segx - gsen2).round(decimals=6)
+            segx1 = (segx + gsen2).round(decimals=6)
+            
+            #fill vectors with interpolated coordinates of start and end points
+            xstartvec[counter:counter+nseg] = np.interp(segx0, L, x)
+            xendvec[counter:counter+nseg] = np.interp(segx1, L, x)
+            
+            ystartvec[counter:counter+nseg] = np.interp(segx0, L, y)
+            yendvec[counter:counter+nseg] = np.interp(segx1, L, y)
+            
+            zstartvec[counter:counter+nseg] = np.interp(segx0, L, z)
+            zendvec[counter:counter+nseg] = np.interp(segx1, L, z)
+                        
+            #fill in values area, diam, length
+            for i, seg in enumerate(sec):
                 areavec[counter] = neuron.h.area(seg.x)
                 diamvec[counter] = seg.diam
-                lengthvec[counter] = sec.L/sec.nseg
-                
-                xstartvec[counter] = neuron.h.x3d(0) + \
-                    xlength * (seg.x - 1./2./sec.nseg)
-                xendvec[counter] = neuron.h.x3d(0) + \
-                    xlength * (seg.x + 1./2./sec.nseg)
-                
-                ystartvec[counter] = neuron.h.y3d(0) + \
-                    ylength * (seg.x - 1./2./sec.nseg)
-                yendvec[counter] = neuron.h.y3d(0) + \
-                    ylength * (seg.x + 1./2./sec.nseg)
-                
-                zstartvec[counter] = neuron.h.z3d(0) + \
-                    zlength * (seg.x - 1./2./sec.nseg)
-                zendvec[counter] = neuron.h.z3d(0) + \
-                    zlength * (seg.x + 1./2./sec.nseg)
-                
+                lengthvec[counter] = sec.L/nseg
+
                 counter += 1
+
+    ##loop over all segments
+    #for sec in cell.allseclist:
+    #    n3d = neuron.h.n3d()
+    #    if n3d > 0:
+    #        #length of sections
+    #        x3d = neuron.h.x3d(0)
+    #        y3d = neuron.h.y3d(0)
+    #        z3d = neuron.h.z3d(0)
+    #        xlength = neuron.h.x3d(n3d - 1) - x3d
+    #        ylength = neuron.h.y3d(n3d - 1) - y3d
+    #        zlength = neuron.h.z3d(n3d - 1) - z3d
+    #        
+    #        secL = sec.L
+    #        nseg = sec.nseg
+    #        gsen2 = 1. / 2. / nseg
+    #        
+    #        for seg in sec:
+    #            segx = seg.x
+    #            
+    #            areavec[counter] = neuron.h.area(segx)
+    #            diamvec[counter] = seg.diam
+    #            lengthvec[counter] = secL / nseg
+    #            
+    #            xstartvec[counter] = x3d + xlength * (segx - gsen2)
+    #            xendvec[counter] = x3d + xlength * (segx + gsen2)
+    #            
+    #            ystartvec[counter] = y3d + ylength * (segx - gsen2)
+    #            yendvec[counter] = y3d + ylength * (segx + gsen2)
+    #            
+    #            zstartvec[counter] = z3d + zlength * (segx - gsen2)
+    #            zendvec[counter] = z3d + zlength * (segx + gsen2)
+    #            
+    #            counter += 1
     
+    #set cell attributes
+    cell.xstart = xstartvec
+    cell.ystart = ystartvec
+    cell.zstart = zstartvec
     
-    cell.xstart = np.array(xstartvec)
-    cell.ystart = np.array(ystartvec)
-    cell.zstart = np.array(zstartvec)
+    cell.xend = xendvec
+    cell.yend = yendvec
+    cell.zend = zendvec
     
-    cell.xend = np.array(xendvec)
-    cell.yend = np.array(yendvec)
-    cell.zend = np.array(zendvec)
-    
-    cell.area = np.array(areavec)
-    cell.diam = np.array(diamvec)
-    cell.length = np.array(lengthvec)
+    cell.area = areavec
+    cell.diam = diamvec
+    cell.length = lengthvec
 
